#include "bussim_arc.h"
#include "bussim_global.h"
#include "math.h"
#include <iostream>
#include <stdio.h>
#
#include <GL/gl.h>
#include <freeglut/freeglut.h>

BUSSIM_ARC::BUSSIM_ARC()
{
    this->ID = -1;

    this->orig_node = -1;
    this->dest_node = -1;
    this->orig_screen_x = -1;
    this->orig_screen_y = -1;
    this->dest_screen_x = -1;
    this->dest_screen_y = -1;

    this->length_screen = 0.0;
    this->pos = -1;

    this->first_line = nullptr;
    this->last_line = nullptr;
    this->lines_on_arc = 0;

    this->type = -1;
}

BUSSIM_ARC::BUSSIM_ARC(int _ID, int _ORIGIN, int _DESTINATION, double _ORIG_SCREEN_X, double _ORIG_SCREEN_Y, double _DEST_SCREEN_X, double _DEST_SCREEN_Y)
{
    this->ID = _ID;
    this->orig_node = _ORIGIN;
    this->dest_node = _DESTINATION;
    this->orig_screen_x = _ORIG_SCREEN_X;
    this->orig_screen_y = _ORIG_SCREEN_Y;
    this->dest_screen_x = _DEST_SCREEN_X;
    this->dest_screen_y = _DEST_SCREEN_Y;
    this->pos = -1;

    this->first_line = nullptr;
    this->last_line = nullptr;
    this->lines_on_arc = 0;
    //this->length = 0.0;
    this->length_screen = 0.0;

    this->type = -1;
}

BUSSIM_ARC::~BUSSIM_ARC()
{

}

void BUSSIM_ARC::update_length(void)
{
    //this->length = sqrt(pow(this->dest_x -this->orig_x,2)+pow(this->dest_y-this->orig_y,2));
    this->length_screen = sqrt(pow(this->dest_screen_x-this->orig_screen_x,2)+pow(this->dest_screen_y-this->orig_screen_y,2));
}

void BUSSIM_ARC::print_on_screen(void)
{


    // print the line
    glBegin(GL_LINES);
            glLineWidth((GLfloat)30);
            glVertex2f(this->orig_screen_x, this->orig_screen_y);
            glVertex2f(this->dest_screen_x, this->dest_screen_y);
    glEnd();


    // print the arc num
    /*
    glRasterPos2d(this->orig_screen_x + 0.1* (this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 0.1* (this->dest_screen_y-this->orig_screen_y));

    char S[256];
    sprintf(S,"NUM=%d",this->ID);
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);
    */

    double diff_x, diff_orth_x;
    double diff_y, diff_orth_y;

    diff_x = this->dest_screen_x - this->orig_screen_x;
    diff_y = this->dest_screen_y - this->orig_screen_y;

    double length_diff;
    length_diff = sqrt(pow(diff_x,2)+pow(diff_y,2));

    diff_orth_x = diff_y;
    diff_orth_y = -diff_x;

    double length_orth;
    length_orth = sqrt(pow(diff_orth_x,2)+pow(diff_orth_y,2));

    glBegin(GL_LINES);
            glVertex2f(this->orig_screen_x + 0.9*(this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 0.9*(this->dest_screen_y-this->orig_screen_y));
            glVertex2f(this->orig_screen_x + 0.9*(this->dest_screen_x-this->orig_screen_x) - 0.01*(this->dest_screen_x-this->orig_screen_x)/length_diff +0.01*diff_orth_x/length_orth, this->orig_screen_y + 0.9*(this->dest_screen_y-this->orig_screen_y)- 0.01*(this->dest_screen_y-this->orig_screen_y)/length_diff+0.01*diff_orth_y/length_orth);
    glEnd();


    glBegin(GL_LINES);
            glVertex2f(this->orig_screen_x + 0.9*(this->dest_screen_x-this->orig_screen_x), this->orig_screen_y + 0.9*(this->dest_screen_y-this->orig_screen_y));
            glVertex2f(this->orig_screen_x + 0.9*(this->dest_screen_x-this->orig_screen_x) - 0.01*(this->dest_screen_x-this->orig_screen_x)/length_diff -0.01*diff_orth_x/length_orth, this->orig_screen_y + 0.9*(this->dest_screen_y-this->orig_screen_y)- 0.01*(this->dest_screen_y-this->orig_screen_y)/length_diff-0.01*diff_orth_y/length_orth);
   glEnd();


}

void BUSSIM_ARC::append_line(class BUSSIM_ARC_IN_LINE *_Ptr)
{
    if( _Ptr != nullptr )
    {
        if( this->lines_on_arc < 1 )
        {
            // add the first line to this arc
            this->first_line = _Ptr;
            this->last_line = _Ptr;
            this->lines_on_arc++;
        }
        else
        {
            // add another line to this arc
            this->last_line->next = _Ptr;
            this->last_line = _Ptr;
            this->lines_on_arc++;
        }
    }
}

void BUSSIM_ARC::set_direction_type(void)
{
    if( this->orig_screen_x < this->dest_screen_x )
        if( this->orig_screen_y < this->dest_screen_y)
            this->type = BUSSIM_NORTHEAST;
        else
            if( this->orig_screen_y > this->dest_screen_y)
                this->type = BUSSIM_SOUTHEAST;
            else
                this->type = BUSSIM_EAST;
    else
        if( this->orig_screen_x > this->dest_screen_x )
            if( this->orig_screen_y < this->dest_screen_y)
                this->type = BUSSIM_NORTHWEST;
            else
                if( this->orig_screen_y > this->dest_screen_y)
                    this->type = BUSSIM_SOUTHWEST;
                else
                    this->type = BUSSIM_WEST;
        else
            if( this->orig_screen_y < this->dest_screen_y)
                this->type = BUSSIM_NORTH;
            else
                this->type = BUSSIM_SOUTH;
}
