#ifndef BUSSIM_GLOBAL_H_INCLUDED
#define BUSSIM_GLOBAL_H_INCLUDED

#include "bussim_network.h"
#include "bussim_vehicle.h"
#include "bussim_arc.h"
#include "bussim_bunchpoint.h"
#include "bussim_event.h"
#include "bussim_line.h"
#include "bussim_point.h"

#define BUSSIM_NO_EVENT -1
#define BUSSIM_ARRIVAL 0
#define BUSSIM_DEPARTURE 1
#define BUSSIM_VEHICLE_ACTIVATION 2
#define BUSSIM_VEHICLE_DEACTIVATION 3
#define BUSSIM_VEHICLE_END_OF_WORK 4
#define BUSSIM_VEHICLE_BLOCKAGE 5

#define BUSSIM_END_OF_DUTY 1440// number of minutes of the working day / start time of densing the SERVICES_ACTIVE_DATABASE

#define BUSSIM_TIME_STEP 0.1               // time offset in every iteration

#define BUSSIM_INITIAL_SPEEDUP_FACTOR 10    // this is the initially used speed factor to be modified during the simulation by the keys <f> (acceleration) and <s> (deceleration)

#define BUSSIM_RANDOM_SEED 0

#define BUSSIM_MEAN_WAITING_TIME 0.75   // mean value of the waiting time of a vehicle at a stop
#define BUSSIM_DEVIATION_WAITING_TIME 0.0   // interval around BUSSIM_STANDARD_WAITING_TIME from which a waiting time at a stop of a vehicle is drawn uniformly distributed

#define BUSSIM_SHOW_LINE_AGENDA 0       // displays the list of lines on the left screen edge if = 1
#define BUSSIM_SHOW_ACTIVE_VEHICLES 0   // displays the list of active vehicles on the right screen edge if = 1
#define BUSSIM_SHOW_BUNCHES 0           // displays bunchpoints if = 1

#define BUSSIM_WINDOWS_SIZE_X 1000      // width of simulation window (in pixels)
#define BUSSIM_WINDOWS_SIZE_Y 1000      // heigth of simulation window (in pixels)

#define BUSSIM_MENUE_POSITION 0.95     // position of menue

#define BUSSIM_NORTH 0
#define BUSSIM_NORTHEAST 1
#define BUSSIM_EAST 2
#define BUSSIM_SOUTHEAST 3
#define BUSSIM_SOUTH 4
#define BUSSIM_SOUTHWEST 5
#define BUSSIM_WEST 6
#define BUSSIM_NORTHWEST 7

#define BUSSIM_REGULARLINE 1
#define BUSSIM_EXTRALINE 2

#define BUSSIM_NODE_LINE 1
#define BUSSIM_NODE_SERVICE 2

// windows definition functions
void set_simulation_window(void);

// opengl callback function definition
void set_callback_funcs(void);

// the prototypes of the opengl callback functions
void display(void);
void tasten( unsigned char key, int x, int y );
void time_loop(int passed_value);

// the menue function
void create_menue(void);
void menu(int id);
void menu_a(int id);
void update_line_list_menue();
#endif // BUSSIM_GLOBAL_H_INCLUDED
