#include "bussim_global.h"
#include "bussim_network.h"
#include "bussim_bunchpoint.h"
#include "bussim_rotation.h"
#include "bussim_duty.h"
#include "bussim_vehicle.h"

#include <math.h>
#include <iostream>
#include <stdio.h>
#include <string>

#include <GL/gl.h>
#include <freeglut/freeglut.h>

BUSSIM_NETWORK::BUSSIM_NETWORK()
{
    // this is the default constructor

    this->POINTS = 0;
    this->ARCS = 0;
    this->LINES = 0;
    this->VEHICLES = 0;
    this->ACTIVE_VEHICLES = 0;
    this->WAITING_VEHICLES = 0;
    this->BUNCHPOINTS = 0;
    this->EVENTS = 0;
    this->DUTIES = 0;

    this->PT = nullptr;
    this->ARC = nullptr;
    this->LINE = nullptr;
    this->VEHICLE = nullptr;
    this->BP = nullptr;

    this->xmin =  1000000.0;
    this->xmax = -1000000.0;
    this->ymin =  1000000.0;
    this->ymax = -1000000.0;

    this->x_factor = 1.0;
    this->y_factor = 1.0;

    this->CURRENT_TIME = 0;
    this->SPEEDUP_FACTOR = BUSSIM_INITIAL_SPEEDUP_FACTOR;
    this->TIME_STEP = BUSSIM_TIME_STEP;

    this->BETRIEBS_ENDE = 0;

    this->DETECTED_BUNCHES = 0;

    this->first_event = nullptr;
    this->last_event = nullptr;

    this->first_bp = nullptr;
    this->last_bp = nullptr;

    this->PRED = nullptr;
}

BUSSIM_NETWORK::BUSSIM_NETWORK(int _POINTS, int _ARCS, int _LINES, int _VEHICLES)
{
  // this is an overloaded constructor

    this->POINTS = _POINTS;
    this->ARCS = _ARCS;
    this->LINES = _LINES;
    this->VEHICLES = _VEHICLES;
    this->ACTIVE_VEHICLES = 0;
    this->WAITING_VEHICLES = 0;
    this->BUNCHPOINTS = 0;
    this->EVENTS = 0;
    this->DUTIES = 0;

    if( this->POINTS > 0)
        this->PT = new class BUSSIM_POINT[this->POINTS];
    if( this->ARCS > 0)
        this->ARC = new class BUSSIM_ARC[this->ARCS];
    if( this->LINES > 0)
        this->LINE = new class BUSSIM_LINE[this->LINES];
    if( this->VEHICLES > 0)
        this->VEHICLE = new class BUSSIM_VEHICLE[this->VEHICLES];
    this->BP = nullptr;

    this->xmin =  1000000.0;
    this->xmax = -1000000.0;
    this->ymin =  1000000.0;
    this->ymax = -1000000.0;

    this->CURRENT_TIME = 0;
    this->SPEEDUP_FACTOR = BUSSIM_INITIAL_SPEEDUP_FACTOR;
    this->TIME_STEP = BUSSIM_TIME_STEP;

    this->BETRIEBS_ENDE = 0;

    this->DETECTED_BUNCHES = 0;

    this->first_event = nullptr;
    this->last_event = nullptr;

    this->first_bp = nullptr;
    this->last_bp = nullptr;

    this->PRED = new int* [this->VEHICLES];
    for( int i=0 ; i < this->VEHICLES ; i++ )
    {
        this->PRED[i] = new int [this->VEHICLES];
        for( int j=0 ; j < this->VEHICLES ; j++ )
            this->PRED[i][j] = 0;
    }

}

BUSSIM_NETWORK::~BUSSIM_NETWORK()
{
    // this is the default constructor
    if ( this->POINTS > 0)
        delete[] this->PT;
    if ( this->ARCS > 0)
        delete[] this->ARC;
    if ( this->LINES > 0)
        delete[] this->LINE;
    if ( this->VEHICLES > 0)
        delete[] this->VEHICLE;
    if ( this->BUNCHPOINTS > 0)
        delete[] this->BP;
}

void BUSSIM_NETWORK::specify_nodes(void)
{
    // here you type in the node information of your network
            // Referenz Haltestelle Hauptbahnhof
    this->PT[0].world_x=0;		    this->PT[0].world_y=0;

    // Haltestellen der Linie 13
    this->PT[1].world_x=4.638;	    this->PT[1].world_y=-4.471;
    this->PT[2].world_x=4.577;	    this->PT[2].world_y=-4.18;
    this->PT[3].world_x=4.66;	    this->PT[3].world_y=-3.815;
    this->PT[4].world_x=5.028;	    this->PT[4].world_y=-3.402;
    this->PT[5].world_x=4.752;	    this->PT[5].world_y=-3.249;
    this->PT[6].world_x=4.389;	    this->PT[6].world_y=-2.954;
    this->PT[7].world_x=4.072;	    this->PT[7].world_y=-2.898;
    this->PT[8].world_x=3.833;	    this->PT[8].world_y=-2.672;
    this->PT[9].world_x=3.571;	    this->PT[9].world_y=-2.433;
    this->PT[10].world_x=3.331;	    this->PT[10].world_y=-2.213;
    this->PT[11].world_x=2.933;	    this->PT[11].world_y=-1.89;
    this->PT[12].world_x=2.553;	    this->PT[12].world_y=-1.672;
    this->PT[13].world_x=2.25;	    this->PT[13].world_y=-1.878;
    this->PT[14].world_x=2.021;	    this->PT[14].world_y=-1.636;
    this->PT[15].world_x=1.873;	    this->PT[15].world_y=-1.354;
    this->PT[16].world_x=1.996;	    this->PT[16].world_y=-0.908;
    this->PT[17].world_x=1.747;	    this->PT[17].world_y=-0.544;
    this->PT[18].world_x=1.39;	    this->PT[18].world_y=-0.334;
    this->PT[19].world_x=1.038;	    this->PT[19].world_y=-0.209;
    this->PT[20].world_x=1.304;	    this->PT[20].world_y=0.362;
    this->PT[21].world_x=1.556;	    this->PT[21].world_y=0.788;
    this->PT[22].world_x=1.706;	    this->PT[22].world_y=1.169;
    this->PT[23].world_x=1.746;	    this->PT[23].world_y=1.359;
    this->PT[24].world_x=1.72;	    this->PT[24].world_y=1.634;
    this->PT[25].world_x=1.383;	    this->PT[25].world_y=2.205;
    this->PT[26].world_x=1.384;	    this->PT[26].world_y=2.588;
    this->PT[27].world_x=1.428;	    this->PT[27].world_y=2.912;
    this->PT[28].world_x=1.583;	    this->PT[28].world_y=3.359;
    this->PT[29].world_x=1.246;	    this->PT[29].world_y=3.563;
    this->PT[30].world_x=0.962;	    this->PT[30].world_y=3.607;
    this->PT[31].world_x=0.517;	    this->PT[31].world_y=3.776;
    this->PT[32].world_x=-0.031;	this->PT[32].world_y=4.109;
    this->PT[33].world_x=-0.54;	    this->PT[33].world_y=4.136;
    this->PT[34].world_x=-0.826;	this->PT[34].world_y=4.194;
    this->PT[35].world_x=-1.117;	this->PT[35].world_y=4.208;
    this->PT[36].world_x=-1.427;	this->PT[36].world_y=4.553;

    // points specification ends here

    // set the ID
    for(int i=0 ; i < this->POINTS ; i++)
        this->PT[i].ID = i;
}

void BUSSIM_NETWORK::specify_arcs(void)
{
    // type in the arc specifications here
        this->ARC[0].orig_node = 1;	this->ARC[0].dest_node = 2;	this->ARC[0].ID = 0;
    this->ARC[1].orig_node = 2;	this->ARC[1].dest_node = 3;	this->ARC[1].ID = 1;
    this->ARC[2].orig_node = 3;	this->ARC[2].dest_node = 4;	this->ARC[2].ID = 2;
    this->ARC[3].orig_node = 4;	this->ARC[3].dest_node = 5;	this->ARC[3].ID = 3;
    this->ARC[4].orig_node = 5;	this->ARC[4].dest_node = 6;	this->ARC[4].ID = 4;
    this->ARC[5].orig_node = 6;	this->ARC[5].dest_node = 7;	this->ARC[5].ID = 5;
    this->ARC[6].orig_node = 7;	this->ARC[6].dest_node = 8;	this->ARC[6].ID = 6;
    this->ARC[7].orig_node = 8;	this->ARC[7].dest_node = 9;	this->ARC[7].ID = 7;
    this->ARC[8].orig_node = 9;	this->ARC[8].dest_node = 10;	this->ARC[8].ID = 8;
    this->ARC[9].orig_node = 10;	this->ARC[9].dest_node = 11;	this->ARC[9].ID = 9;
    this->ARC[10].orig_node = 11;	this->ARC[10].dest_node = 12;	this->ARC[10].ID = 10;
    this->ARC[11].orig_node = 12;	this->ARC[11].dest_node = 13;	this->ARC[11].ID = 11;
    this->ARC[12].orig_node = 13;	this->ARC[12].dest_node = 14;	this->ARC[12].ID = 12;
    this->ARC[13].orig_node = 14;	this->ARC[13].dest_node = 15;	this->ARC[13].ID = 13;
    this->ARC[14].orig_node = 15;	this->ARC[14].dest_node = 16;	this->ARC[14].ID = 14;
    this->ARC[15].orig_node = 16;	this->ARC[15].dest_node = 17;	this->ARC[15].ID = 15;
    this->ARC[16].orig_node = 17;	this->ARC[16].dest_node = 18;	this->ARC[16].ID = 16;
    this->ARC[17].orig_node = 18;	this->ARC[17].dest_node = 19;	this->ARC[17].ID = 17;
    this->ARC[18].orig_node = 19;	this->ARC[18].dest_node = 20;	this->ARC[18].ID = 18;
    this->ARC[19].orig_node = 20;	this->ARC[19].dest_node = 21;	this->ARC[19].ID = 19;
    this->ARC[20].orig_node = 21;	this->ARC[20].dest_node = 22;	this->ARC[20].ID = 20;
    this->ARC[21].orig_node = 22;	this->ARC[21].dest_node = 23;	this->ARC[21].ID = 21;
    this->ARC[22].orig_node = 23;	this->ARC[22].dest_node = 24;	this->ARC[22].ID = 22;
    this->ARC[23].orig_node = 24;	this->ARC[23].dest_node = 25;	this->ARC[23].ID = 23;
    this->ARC[24].orig_node = 25;	this->ARC[24].dest_node = 26;	this->ARC[24].ID = 24;
    this->ARC[25].orig_node = 26;	this->ARC[25].dest_node = 27;	this->ARC[25].ID = 25;
    this->ARC[26].orig_node = 27;	this->ARC[26].dest_node = 28;	this->ARC[26].ID = 26;
    this->ARC[27].orig_node = 28;	this->ARC[27].dest_node = 29;	this->ARC[27].ID = 27;
    this->ARC[28].orig_node = 29;	this->ARC[28].dest_node = 30;	this->ARC[28].ID = 28;
    this->ARC[29].orig_node = 30;	this->ARC[29].dest_node = 31;	this->ARC[29].ID = 29;
    this->ARC[30].orig_node = 31;	this->ARC[30].dest_node = 32;	this->ARC[30].ID = 30;
    this->ARC[31].orig_node = 32;	this->ARC[31].dest_node = 33;	this->ARC[31].ID = 31;
    this->ARC[32].orig_node = 33;	this->ARC[32].dest_node = 34;	this->ARC[32].ID = 32;
    this->ARC[33].orig_node = 34;	this->ARC[33].dest_node = 35;	this->ARC[33].ID = 33;
    this->ARC[34].orig_node = 35;	this->ARC[34].dest_node = 36;	this->ARC[34].ID = 34;

    this->ARC[35].orig_node = 36;	this->ARC[35].dest_node = 35;	this->ARC[35].ID = 35;
    this->ARC[36].orig_node = 35;	this->ARC[36].dest_node = 34;	this->ARC[36].ID = 36;
    this->ARC[37].orig_node = 34;	this->ARC[37].dest_node = 33;	this->ARC[37].ID = 37;
    this->ARC[38].orig_node = 33;	this->ARC[38].dest_node = 32;	this->ARC[38].ID = 38;
    this->ARC[39].orig_node = 32;	this->ARC[39].dest_node = 31;	this->ARC[39].ID = 39;
    this->ARC[40].orig_node = 31;	this->ARC[40].dest_node = 30;	this->ARC[40].ID = 40;
    this->ARC[41].orig_node = 30;	this->ARC[41].dest_node = 29;	this->ARC[41].ID = 41;
    this->ARC[42].orig_node = 29;	this->ARC[42].dest_node = 28;	this->ARC[42].ID = 42;
    this->ARC[43].orig_node = 28;	this->ARC[43].dest_node = 27;	this->ARC[43].ID = 43;
    this->ARC[44].orig_node = 27;	this->ARC[44].dest_node = 26;	this->ARC[44].ID = 44;
    this->ARC[45].orig_node = 26;	this->ARC[45].dest_node = 25;	this->ARC[45].ID = 45;
    this->ARC[46].orig_node = 25;	this->ARC[46].dest_node = 24;	this->ARC[46].ID = 46;
    this->ARC[47].orig_node = 24;	this->ARC[47].dest_node = 23;	this->ARC[47].ID = 47;
    this->ARC[48].orig_node = 23;	this->ARC[48].dest_node = 22;	this->ARC[48].ID = 48;
    this->ARC[49].orig_node = 22;	this->ARC[49].dest_node = 21;	this->ARC[49].ID = 49;
    this->ARC[50].orig_node = 21;	this->ARC[50].dest_node = 20;	this->ARC[50].ID = 50;
    this->ARC[51].orig_node = 20;	this->ARC[51].dest_node = 19;	this->ARC[51].ID = 51;
    this->ARC[52].orig_node = 19;	this->ARC[52].dest_node = 18;	this->ARC[52].ID = 52;
    this->ARC[53].orig_node = 18;	this->ARC[53].dest_node = 17;	this->ARC[53].ID = 53;
    this->ARC[54].orig_node = 17;	this->ARC[54].dest_node = 16;	this->ARC[54].ID = 54;
    this->ARC[55].orig_node = 16;	this->ARC[55].dest_node = 15;	this->ARC[55].ID = 55;
    this->ARC[56].orig_node = 15;	this->ARC[56].dest_node = 14;	this->ARC[56].ID = 56;
    this->ARC[57].orig_node = 14;	this->ARC[57].dest_node = 13;	this->ARC[57].ID = 57;
    this->ARC[58].orig_node = 13;	this->ARC[58].dest_node = 12;	this->ARC[58].ID = 58;
    this->ARC[59].orig_node = 12;	this->ARC[59].dest_node = 11;	this->ARC[59].ID = 59;
    this->ARC[60].orig_node = 11;	this->ARC[60].dest_node = 10;	this->ARC[60].ID = 60;
    this->ARC[61].orig_node = 10;	this->ARC[61].dest_node = 9;	this->ARC[61].ID = 61;
    this->ARC[62].orig_node = 9;	this->ARC[62].dest_node = 8;	this->ARC[62].ID = 62;
    this->ARC[63].orig_node = 8;	this->ARC[63].dest_node = 7;	this->ARC[63].ID = 63;
    this->ARC[64].orig_node = 7;	this->ARC[64].dest_node = 6;	this->ARC[64].ID = 64;
    this->ARC[65].orig_node = 6;	this->ARC[65].dest_node = 5;	this->ARC[65].ID = 65;
    this->ARC[66].orig_node = 5;	this->ARC[66].dest_node = 4;	this->ARC[66].ID = 66;
    this->ARC[67].orig_node = 4;	this->ARC[67].dest_node = 3;	this->ARC[67].ID = 67;
    this->ARC[68].orig_node = 3;	this->ARC[68].dest_node = 2;	this->ARC[68].ID = 68;
    this->ARC[69].orig_node = 2;	this->ARC[69].dest_node = 1;	this->ARC[69].ID = 69;

    // arc specification ends here

    for(int a=0 ; a < this->ARCS ; a++)
        this->ARC[a].update_length();
}

void BUSSIM_NETWORK::specify_lines(void)
{

    // type in your line specifications here
    this->LINE[0].configure(0,13,1.0,1.0,0.0,"Linie 13: Prohlis->Mickten");
    this->LINE[0].append_arc(this->ARC[0]);
    this->LINE[0].append_arc(this->ARC[1]);
    this->LINE[0].append_arc(this->ARC[2]);
    this->LINE[0].append_arc(this->ARC[3]);
    this->LINE[0].append_arc(this->ARC[4]);
    this->LINE[0].append_arc(this->ARC[5]);
    this->LINE[0].append_arc(this->ARC[6]);
    this->LINE[0].append_arc(this->ARC[7]);
    this->LINE[0].append_arc(this->ARC[8]);
    this->LINE[0].append_arc(this->ARC[9]);
    this->LINE[0].append_arc(this->ARC[10]);
    this->LINE[0].append_arc(this->ARC[11]);
    this->LINE[0].append_arc(this->ARC[12]);
    this->LINE[0].append_arc(this->ARC[13]);
    this->LINE[0].append_arc(this->ARC[14]);
    this->LINE[0].append_arc(this->ARC[15]);
    this->LINE[0].append_arc(this->ARC[16]);
    this->LINE[0].append_arc(this->ARC[17]);
    this->LINE[0].append_arc(this->ARC[18]);
    this->LINE[0].append_arc(this->ARC[19]);
    this->LINE[0].append_arc(this->ARC[20]);
    this->LINE[0].append_arc(this->ARC[21]);
    this->LINE[0].append_arc(this->ARC[22]);
    this->LINE[0].append_arc(this->ARC[23]);
    this->LINE[0].append_arc(this->ARC[24]);
    this->LINE[0].append_arc(this->ARC[25]);
    this->LINE[0].append_arc(this->ARC[26]);
    this->LINE[0].append_arc(this->ARC[27]);
    this->LINE[0].append_arc(this->ARC[28]);
    this->LINE[0].append_arc(this->ARC[29]);
    this->LINE[0].append_arc(this->ARC[30]);
    this->LINE[0].append_arc(this->ARC[31]);
    this->LINE[0].append_arc(this->ARC[32]);
    this->LINE[0].append_arc(this->ARC[33]);
    this->LINE[0].append_arc(this->ARC[34]);

    // Linienverlaufskodierung Linie 13 von Mickten nach Prohlis
    this->LINE[1].configure(1,13,1.0,1.0,0.0,"Linie 13: Mickten->Prohlis");
    this->LINE[1].append_arc(this->ARC[35]);
    this->LINE[1].append_arc(this->ARC[36]);
    this->LINE[1].append_arc(this->ARC[37]);
    this->LINE[1].append_arc(this->ARC[38]);
    this->LINE[1].append_arc(this->ARC[39]);
    this->LINE[1].append_arc(this->ARC[40]);
    this->LINE[1].append_arc(this->ARC[41]);
    this->LINE[1].append_arc(this->ARC[42]);
    this->LINE[1].append_arc(this->ARC[43]);
    this->LINE[1].append_arc(this->ARC[44]);
    this->LINE[1].append_arc(this->ARC[45]);
    this->LINE[1].append_arc(this->ARC[46]);
    this->LINE[1].append_arc(this->ARC[47]);
    this->LINE[1].append_arc(this->ARC[48]);
    this->LINE[1].append_arc(this->ARC[49]);
    this->LINE[1].append_arc(this->ARC[50]);
    this->LINE[1].append_arc(this->ARC[51]);
    this->LINE[1].append_arc(this->ARC[52]);
    this->LINE[1].append_arc(this->ARC[53]);
    this->LINE[1].append_arc(this->ARC[54]);
    this->LINE[1].append_arc(this->ARC[55]);
    this->LINE[1].append_arc(this->ARC[56]);
    this->LINE[1].append_arc(this->ARC[57]);
    this->LINE[1].append_arc(this->ARC[58]);
    this->LINE[1].append_arc(this->ARC[59]);
    this->LINE[1].append_arc(this->ARC[60]);
    this->LINE[1].append_arc(this->ARC[61]);
    this->LINE[1].append_arc(this->ARC[62]);
    this->LINE[1].append_arc(this->ARC[63]);
    this->LINE[1].append_arc(this->ARC[64]);
    this->LINE[1].append_arc(this->ARC[65]);
    this->LINE[1].append_arc(this->ARC[66]);
    this->LINE[1].append_arc(this->ARC[67]);
    this->LINE[1].append_arc(this->ARC[68]);
    this->LINE[1].append_arc(this->ARC[69]);
 }

void BUSSIM_NETWORK::specify_vehicles(void)
{
    // this procedure is the place where the vehicle specifications (ID as well as velocity) are specified

    // specify the available vehicles here

    this->VEHICLE[0].configure(0,25,this);
    this->VEHICLE[1].configure(1,25,this);
    this->VEHICLE[2].configure(2,25,this);
    this->VEHICLE[3].configure(3,25,this);
    this->VEHICLE[4].configure(4,25,this);
    this->VEHICLE[5].configure(5,25,this);
    this->VEHICLE[6].configure(6,25,this);
    this->VEHICLE[7].configure(7,25,this);
    this->VEHICLE[8].configure(8,25,this);
    this->VEHICLE[9].configure(9,25,this);


}

void BUSSIM_NETWORK::specify(void)
{
    // this procedure is used to define the network

    // define the network nodes
    this->specify_nodes();

    // specify the arcs
    this->specify_arcs();

    // determine extreme coordinates
   for(int j=0 ; j < this->POINTS ; j++)
        {
            if( this->PT[j].world_x < this->xmin )
                this->xmin = this->PT[j].world_x;
            if( this->PT[j].world_x > this->xmax )
                this->xmax = this->PT[j].world_x;
            if( this->PT[j].world_y < this->ymin )
                this->ymin = this->PT[j].world_y;
            if( this->PT[j].world_y > this->ymax )
                this->ymax = this->PT[j].world_y;
        }

    // ensure that even the outer points are printed completely
    double x_os;
    x_os = (this->xmax - this->xmin) / 20.0;
    double y_os;
    y_os = (this->ymax - this->ymin) / 20.0;

    this->xmax = this->xmax+x_os;
    this->ymax = this->ymax+y_os;
    this->xmin = this->xmin-x_os;
    this->ymin = this->ymin-y_os;

    // set the screen coordinates
    for(int j=0 ; j < this->POINTS ; j++)
        this->PT[j].calculate_screen_coordinates(this,this->xmin, this->xmax, this->ymin, this->ymax);

    this->center_network();

    // transfer the screen coordinates from points into arcs
    for( int i=0 ; i < this->ARCS ; i++ )
    {
            this->ARC[i].orig_screen_x = this->PT[this->ARC[i].orig_node].screen_x;
            this->ARC[i].orig_screen_y = this->PT[this->ARC[i].orig_node].screen_y;
            this->ARC[i].dest_screen_x = this->PT[this->ARC[i].dest_node].screen_x;
            this->ARC[i].dest_screen_y = this->PT[this->ARC[i].dest_node].screen_y;
            this->ARC[i].update_length();
    }

    // specify the arc types
    for(int a=0 ; a < this->ARCS ; a++ )
            this->ARC[a].set_direction_type();


    // specify the lines
    this->specify_lines();

    // specify the vehicles
    this->specify_vehicles();

    // specify the vehicle rotations
    this->specify_rotations();

    // analyze the lines
    this->merge_lines_arcs();

}

void BUSSIM_NETWORK::display_active_vehicles(void)
{

    glRasterPos2d(0.7, 0.95);

    char S[256];
    sprintf(S,"active vehicles: ");

    glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
    glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
    double y_offset = 0.05;
    for(int i=0 ; i < this->VEHICLES ; i++ )
    {
        if( this->VEHICLE[i].ACTIVE > 0 )
            if( this->VEHICLE[i].VISIBLE > 0)
        {
             glColor4f((float)(this->LINE[this->VEHICLE[i].on_line].RED), (float)(this->LINE[this->VEHICLE[i].on_line].GREEN), (float)(this->LINE[this->VEHICLE[i].on_line].BLUE), 0.0f);
           glRasterPos2d(0.6,0.95 - y_offset);

            //char S[256];
            if( this->VEHICLE[i].Thwarted < -10 )
                sprintf(S,"V%d on arc %d (blocked %d)",this->VEHICLE[i].ID, this->VEHICLE[i].ON_ARC.ID, this->VEHICLE[i].Thwarted);
            else
                sprintf(S,"V%d on arc %d (%.2f) rot. %ld",this->VEHICLE[i].ID, this->VEHICLE[i].ON_ARC.ID, this->VEHICLE[i].last_node_arrival_time,this->VEHICLE[i].rotation->current_duty->ID);
            unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
            glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
            //std:: cout << this->LINE[this->VEHICLE[i].on_line].RED << "-" << this->LINE[this->VEHICLE[i].on_line].GREEN << "-" << this->LINE[this->VEHICLE[i].on_line].BLUE << std::endl;
            //glColor4f(1.0f, 1.0f, 1.0f, 0.0f);
            y_offset = y_offset + 0.03;
        }
    }
}

void BUSSIM_NETWORK::start_rotations(void)
{

    if( (this->CURRENT_TIME >= 0) && (this->CURRENT_TIME < 0.5) )
    {
        this->VEHICLE[0].activate(this);
    }
    if( (this->CURRENT_TIME >= 10) && (this->CURRENT_TIME < 10.5) )
    {
        this->VEHICLE[1].activate(this);
    }
    if( (this->CURRENT_TIME >= 20) && (this->CURRENT_TIME < 20.5) )
    {
        this->VEHICLE[2].activate(this);
    }
    if( (this->CURRENT_TIME >= 0) && (this->CURRENT_TIME < 0.5) )
    {
        this->VEHICLE[3].activate(this);
    }
    if( (this->CURRENT_TIME >= 10) && (this->CURRENT_TIME < 10.5) )
    {
        this->VEHICLE[4].activate(this);
    }
    if( (this->CURRENT_TIME >= 20) && (this->CURRENT_TIME < 20.5) )
    {
        this->VEHICLE[5].activate(this);
    }
}

void BUSSIM_NETWORK::append_bunch_point(class BUSSIM_BUNCHPOINT *_POINT)
{
    int check = 1;

    if( this->BUNCHPOINTS > 0 )
    {
        // first check if this point is already saved
        class BUSSIM_BUNCHPOINT *TmpPtr;
        TmpPtr = this->first_bp;
        while( TmpPtr != nullptr )
        {
            if( ( fabs(TmpPtr->screen_x - _POINT->screen_x) < 0.0001)
               && ( fabs(TmpPtr->screen_y - _POINT->screen_y) < 0.0001) )
            {
                // recent point is already saved
                TmpPtr->num_of_appear++;
                check = 0;
                TmpPtr = nullptr;
                this->BUNCHPOINTS++;
            }
            else
            {
                // not yet found, go to next
                TmpPtr = TmpPtr->next;
            }
        }
        //std::cout << "hinzufuegen" << std::endl;
        if( check == 1)
        {
            // we have to create a new bunchpoint-object
            class BUSSIM_BUNCHPOINT *AddBP;
            AddBP = new class BUSSIM_BUNCHPOINT(_POINT->screen_x,_POINT->screen_y);
            AddBP->previous = this->last_bp;
            this->last_bp->next = AddBP;
            this->last_bp = AddBP;
            AddBP = nullptr;
            this->BUNCHPOINTS++;
        }
        //std::cout << "hinzufuegen beendet" << std::endl;
    }
    else
    {
        //std::cout << "1. hinzufuegen" << std::endl;
        class BUSSIM_BUNCHPOINT *AddBP;
        AddBP = new class BUSSIM_BUNCHPOINT(_POINT->screen_x,_POINT->screen_y);
        this->first_bp = AddBP;
        this->last_bp = AddBP;
        AddBP = nullptr;
        this->BUNCHPOINTS++;
        //std::cout << "1. hinzufuegen beendet" << std::endl;
    }


}

void BUSSIM_NETWORK::print_bunch_points(void)
{
    float Color;

    class BUSSIM_BUNCHPOINT *TmpPtr;

    TmpPtr = this->first_bp;

    while( TmpPtr != nullptr)
    {
        double r = 0.01+0.00; //*this->BP[i].label_pos;

        int num_segments = 1000;

        glBegin(GL_LINE_LOOP);
        Color = 0.3f + (float)((TmpPtr->num_of_appear)/((double)(this->BUNCHPOINTS))*0.7);
        glColor4f(Color , Color, Color, 0.0f);
        for(int ii = 0; ii < num_segments; ii++)
        {
            float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

            float x = r * cosf(theta);//calculate the x component
            float y = r * sinf(theta);//calculate the y component

            glVertex2f(TmpPtr->screen_x, TmpPtr->screen_y);
            glVertex2f(x + TmpPtr->screen_x, y + TmpPtr->screen_y);//output vertex

        }
        glEnd();
        TmpPtr = TmpPtr->next;
    }

}

void BUSSIM_NETWORK::append_event(class BUSSIM_EVENT *_EVENT)
{
    // this procedure appends an event object to the list of previously added event objects
    if( this->first_event == nullptr )
    {
        // event list is empty
        this->first_event = _EVENT;
        _EVENT->previous = nullptr;
        this->last_event = _EVENT;
        _EVENT->next = nullptr;
        this->EVENTS++;
    }
    else
    {
        // event list already contains some events objects
        _EVENT->previous = this->last_event;
        _EVENT->previous->next = _EVENT;
        this->last_event = _EVENT;
        this->EVENTS++;
    }
}

void BUSSIM_NETWORK::print_event_list(void)
{
    class BUSSIM_EVENT *TmpPtr;

    TmpPtr = this->first_event;

    while( TmpPtr != nullptr )
    {
        TmpPtr->PRINT_ON_SCREEN();
        TmpPtr = TmpPtr->next;
    }
}

int BUSSIM_NETWORK::number_of_active_vehicles(void)
{
    int ret_value = 0;
    for(int v=0 ; v < this->VEHICLES ; v++)
    {
        ret_value += this->VEHICLE[v].ACTIVE;
    }

    return ret_value;
}

void BUSSIM_NETWORK::center_network(void)
{
    // this procedure corrects the point positions of the screen to center the network presentation

    double x_min_nodes = 1000.0;
    double x_max_nodes = -1000.0;
    double x_middle_nodes;
    double y_min_nodes = 1000.0;
    double y_max_nodes = -1000.0;
    double y_middle_nodes;
    for(int n=0 ; n < this->POINTS ; n++)
    {
        if( this->PT[n].screen_x < x_min_nodes )
            x_min_nodes = this->PT[n].screen_x;
        if( this->PT[n].screen_x > x_max_nodes )
            x_max_nodes = this->PT[n].screen_x;
        if( this->PT[n].screen_y < y_min_nodes )
            y_min_nodes = this->PT[n].screen_y;
        if( this->PT[n].screen_y > y_max_nodes )
            y_max_nodes = this->PT[n].screen_y;
    }

    x_middle_nodes = x_min_nodes + (x_max_nodes - x_min_nodes) / 2.0;
    y_middle_nodes = y_min_nodes + (y_max_nodes - y_min_nodes) / 2.0;

    if( x_middle_nodes < 0 )
    {
        // move everything a little bit up
        for(int n=0 ; n < this->POINTS ; n++)
        {
            this->PT[n].screen_x -= x_middle_nodes;
        }
    }
    if( x_middle_nodes > 0 )
    {
        // move everything a little bit down
        for(int n=0 ; n < this->POINTS ; n++)
        {
            this->PT[n].screen_x += x_middle_nodes;
        }

    }
    if( y_middle_nodes < 0 )
    {
        // move everything a little bit up
        for(int n=0 ; n < this->POINTS ; n++)
        {
            this->PT[n].screen_y -= y_middle_nodes;
        }
    }
    if( y_middle_nodes > 0 )
    {
        // move everything a little bit down
        for(int n=0 ; n < this->POINTS ; n++)
        {
            this->PT[n].screen_y += y_middle_nodes;
        }

    }
}

void BUSSIM_NETWORK::merge_lines_arcs(void)
{
    // go through all lines
    for( int l=0 ; l < this->LINES ; l++ )
    {
        // now run through the arcs of the current line
        for( int a=0 ; a < this->LINE[l].ARCS ; a++ )
        {
            class BUSSIM_ARC_IN_LINE *Tmp;
            Tmp = new class BUSSIM_ARC_IN_LINE;
            Tmp->LINE_ID = l;
            Tmp->next = nullptr;
            if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_x
               < this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_x )
            {
                // left to right
                if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                        < this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                        {
                            // to the north east
                            Tmp->x_offset = 0.05;
                            Tmp->y_offset = -0.05;
                        }
                        else
                            if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                                > this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                                {
                                    // to the south east
                                    Tmp->x_offset = -0.05;
                                    Tmp->y_offset = -0.05;
                                }
                                else
                                {
                                    // to the east
                                    Tmp->x_offset = 0.0;
                                    Tmp->y_offset = -0.05;
                                }

            }
            else
                if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_x
               > this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_x )
                {
                    // right to left
                     if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                        < this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                        {
                            // to the northwest
                            Tmp->x_offset = 0.05;
                            Tmp->y_offset = 0.05;
                        }
                        else
                            if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                            > this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                            {
                                // to the northeast
                                Tmp->x_offset = -0.05;
                                Tmp->y_offset = +0.05;
                            }
                            else
                            {
                                // to the west
                                Tmp->x_offset = 0.0;
                                Tmp->y_offset = +0.05;
                            }
                }
                else
                {
                    // vertical
                    if( this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].orig_node].screen_y
                        < this->PT[this->ARC[this->LINE[l].USED_ARC[a].ID].dest_node].screen_y )
                    {
                        // to the north
                        Tmp->x_offset = 0.05;
                        Tmp->y_offset = 0;
                    }
                    else
                    {
                        // to the south
                        Tmp->x_offset = 0;
                        Tmp->y_offset = -0.05;
                    }
                }

            // consider only regularlines
            if( this->LINE[l].type == BUSSIM_REGULARLINE )
                this->ARC[this->LINE[l].USED_ARC[a].ID].append_line(Tmp);
            Tmp = nullptr;
        }
    }

    //exit(0);
}

void BUSSIM_NETWORK::set_visibility(void)
{
    // here the visibility of the available vehicles can be specified

    // make all vehicles invisible

    for( int v=0 ; v < this->VEHICLES ; v++ )
    {
        this->VEHICLE[v].VISIBLE = 1;
    }


    // make selected vehicle visisble again

    for( int l=0 ; l < this->LINES ; l++ )
    {
        this->LINE[l].displayed = 1;
    }

}

void BUSSIM_NETWORK::draw_depots(void)
{
    for( int p=0 ; p < this->POINTS ; p++ )
    {
        if( this->PT[p].type == BUSSIM_NODE_SERVICE )
        {
            glBegin(GL_POLYGON);
            glVertex2f(this->PT[p].screen_x-0.01, this->PT[p].screen_y+0.01);
            glVertex2f(this->PT[p].screen_x+0.01, this->PT[p].screen_y+0.01);
            glVertex2f(this->PT[p].screen_x+0.01, this->PT[p].screen_y-0.01);
            glVertex2f(this->PT[p].screen_x-0.01, this->PT[p].screen_y-0.01);
            glEnd();
        }
    }
}

void BUSSIM_NETWORK::draw_service_arcs(void)
{
    for( int a=0 ; a < this->ARCS ; a++ )
    {
        if( this->ARC[a].lines_on_arc < 1 )
        {
            glBegin(GL_LINES);
            glLineWidth((GLfloat)30);
            glVertex2f(this->ARC[a].orig_screen_x, this->ARC[a].orig_screen_y);
            glVertex2f(this->ARC[a].dest_screen_x, this->ARC[a].dest_screen_y);
            glEnd();
        }
    }
}

void BUSSIM_NETWORK::append_duty_to_vehicle_rotation(int _VEH_ID, int _TRIP_ID)
{
    // this procedure creates a new duty containing line _TRIP_ID and appends this
    // duty to the rotation of _VEH_ID

    // create the pointer for a new duty
    BUSSIM_DUTY *NEW_DUTY;

    // allocate the memory for the new duty
    NEW_DUTY = new struct BUSSIM_DUTY();

    // assign the unique id to the duty
    NEW_DUTY->ID = this->DUTIES;

    // increase the network-wide duty counter
    this->DUTIES++;

    // assign the trip id
    NEW_DUTY->TRIP_ID = _TRIP_ID;

    // append the newly created duty to the specified vehicle's rotation
    this->VEHICLE[_VEH_ID].rotation->append_duty(NEW_DUTY);

    // disconnect the pointer from the created duty
    NEW_DUTY = nullptr;
}

void BUSSIM_NETWORK::specify_rotations(void)
{
    // this procedure defines the vehicle rotations

    this->append_duty_to_vehicle_rotation(0,0);
    this->append_duty_to_vehicle_rotation(0,1);
    this->append_duty_to_vehicle_rotation(0,0);
    this->append_duty_to_vehicle_rotation(0,1);

    this->append_duty_to_vehicle_rotation(1,1);
    this->append_duty_to_vehicle_rotation(1,0);
    this->append_duty_to_vehicle_rotation(1,1);
    this->append_duty_to_vehicle_rotation(1,0);

    this->append_duty_to_vehicle_rotation(2,0);
    this->append_duty_to_vehicle_rotation(2,1);
    this->append_duty_to_vehicle_rotation(2,0);

    this->append_duty_to_vehicle_rotation(3,1);
    this->append_duty_to_vehicle_rotation(3,0);
    this->append_duty_to_vehicle_rotation(3,1);
    this->append_duty_to_vehicle_rotation(3,0);
    this->append_duty_to_vehicle_rotation(3,1);
}
