#include "bussim_point.h"
#include "bussim_network.h"
#include "bussim_global.h"

#include <math.h>           // needed to draw the circle
#include <iostream>
#include <stdio.h>
#include <string.h>

#include <GL/gl.h>
#include <freeglut/freeglut.h>

BUSSIM_POINT::BUSSIM_POINT()
{
    // this is the source code of the standard constructor
    // it sets both coordinates to -1

    this->ID = -1;
    this->label_pos = 1;
    this->world_x = -1.0;
    this->world_y = -1.0;
    this->screen_x = -1.0;
    this->screen_y = -1.0;
    this->type = BUSSIM_NODE_LINE;
}

BUSSIM_POINT::BUSSIM_POINT(double _X, double _Y)
{
    // this is the source code of a first overloaded constructor
    // it sets the coordinates to the specified values

    this->ID = -1;
    this->label_pos = 1;
    this->world_x = _X;
    this->world_y = _Y;
    this->screen_x = -1.0;
    this->screen_y = -1.0;
    this->type = BUSSIM_NODE_LINE;
}

BUSSIM_POINT::~BUSSIM_POINT()
{
    // this is the source code of the standard destructor
    // it does nothing (but should be there)
}

void BUSSIM_POINT::calculate_screen_coordinates(class BUSSIM_NETWORK *_NET, double _xmin, double _xmax, double _ymin, double _ymax)
{
    if( (_ymax - _ymin) > (_xmax - _xmin) )
    {
        _NET->y_factor = 1;
        _NET->x_factor = (_xmax - _xmin) / (_ymax - _ymin);
    }
    else
    {
        _NET->x_factor = 1;
        _NET->y_factor = (_ymax - _ymin) / (_xmax - _xmin);
    }

    this->screen_x = -1 + (this->world_x-_xmin)/(_xmax-_xmin)*2*_NET->x_factor;
    this->screen_y = -1 + (this->world_y-_ymin)/(_ymax-_ymin)*2*_NET->y_factor;

}

void BUSSIM_POINT::print_on_screen(void)
{
  double r = 0.030;
    int num_segments = 1000;

    glBegin(GL_LINE_LOOP);
    //glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
    for(int ii = 0; ii < num_segments; ii++)
    {
        float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

        float x = r * cosf(theta);//calculate the x component
        float y = r * sinf(theta);//calculate the y component

        glVertex2f(this->screen_x, this->screen_y);
        glVertex2f(x + this->screen_x, y + this->screen_y);//output vertex

    }
    glEnd();

}

void BUSSIM_POINT::print_on_screen_small(void)
{
  double r = 0.010;
    int num_segments = 1000;

    glBegin(GL_LINE_LOOP);
    //glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
    for(int ii = 0; ii < num_segments; ii++)
    {
        float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

        float x = r * cosf(theta);//calculate the x component
        float y = r * sinf(theta);//calculate the y component

        glVertex2f(this->screen_x, this->screen_y);
        glVertex2f(x + this->screen_x, y + this->screen_y);//output vertex

    }
    glEnd();

}

void BUSSIM_POINT::print_label_on_screen(void)
{
    glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
    if( this->label_pos == 0 )
        glRasterPos2d(this->screen_x, this->screen_y);
    if( this->label_pos == 1 )
        glRasterPos2d(this->screen_x+0.01, this->screen_y+0.01);
    if( this->label_pos == 2 )
        glRasterPos2d(this->screen_x, this->screen_y+0.01);
    if( this->label_pos == 3 )
        glRasterPos2d(this->screen_x-0.01, this->screen_y+0.01);
    if( this->label_pos == 4 )
        glRasterPos2d(this->screen_x-0.01, this->screen_y);
    if( this->label_pos == 5 )
        glRasterPos2d(this->screen_x-0.01, this->screen_y-0.01);
    if( this->label_pos == 6 )
        glRasterPos2d(this->screen_x, this->screen_y-0.01);
    if( this->label_pos == 7 )
        glRasterPos2d(this->screen_x+0.01, this->screen_y-0.01);
    if( this->label_pos == 8 )
        glRasterPos2d(this->screen_x+0.01, this->screen_y);

    //glRasterPos2d(this->screen_x-0.0125, this->screen_y-0.0125);


    char S[256];
    sprintf(S,"%d",this->ID);
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);

    glutBitmapString(GLUT_BITMAP_HELVETICA_12, ubuf);
    //glColor4f(1.0f, 0.0f, 0.0f, 0.0f);
}
