#include "bussim_vehicle.h"
#include "bussim_global.h"
#include "bussim_bunchpoint.h"
#include "bussim_rotation.h"

#include <math.h>
#include <iostream>
#include <stdio.h>

#include <GL/gl.h>
#include <freeglut/freeglut.h>

extern int AUTOMATIC_MODE;

BUSSIM_VEHICLE::BUSSIM_VEHICLE()
{
    this->ID = -1;
    this->on_line = -1;
    this->velocity = -1.0;
    this->ACTIVE = 0;
    this->WAITING = 0;
    this->VISIBLE = 1;

    this->last_update_time = -1.0;

    this->ON_ARC.dest_node = -1;
    this->ON_ARC.orig_node = -1;
    this->ON_ARC.dest_screen_x = -1.0;
    this->ON_ARC.dest_screen_y = -1.0;
    this->ON_ARC.length_screen = -1.0;
    this->ON_ARC.orig_screen_x = -1.0;
    this->ON_ARC.orig_screen_y = -1.0;

    this->POS = -1.0;
    this->remaing_waiting_time = -1.0;

    this->km_travelled_on_line = 0;
    this->start_time_on_line = -1.0;

    this->km_travelled_on_arc = 0;
    this->start_time_on_arc = -1.0;

    this->Thwarted = 0;

    this->rotation = nullptr;

    this->next_arc.ID = -1;     // to be set in the activate function
    this->last_visited_node = -1;     // to be set in the activate function
    this->last_node_arrival_time = -1.0;    // to be set in the activate function
}

void BUSSIM_VEHICLE::configure(int _ID, double _VELOCITY, class BUSSIM_NETWORK *_NET)
{
    this->ID = _ID;
    this->on_line = -1;
    this->velocity = _VELOCITY;
    this->ACTIVE = 0;
    this->WAITING = 0;
    this->VISIBLE = 1;

    this->last_update_time = -1.0;

    this->ON_ARC.dest_node = -1;
    this->ON_ARC.orig_node = -1;
    this->ON_ARC.dest_screen_x = -1.0;
    this->ON_ARC.dest_screen_y = -1.0;
    this->ON_ARC.length_screen = -1.0;
    this->ON_ARC.orig_screen_x = -1.0;
    this->ON_ARC.orig_screen_y = -1.0;

    this->POS = -1.0;
    this->remaing_waiting_time = -1.0;

    this->km_travelled_on_line = 0;
    this->start_time_on_line = -1.0;

    this->km_travelled_on_arc = 0;
    this->start_time_on_arc = -1.0;

    this->Thwarted = 0;

    this->rotation = new struct BUSSIM_ROTATION(_NET->ROTATIONS);
    _NET->ROTATIONS++;;

    this->next_arc.ID = -1;     // to be set in the activate function
    this->last_visited_node = -1;     // to be set in the activate function

}

BUSSIM_VEHICLE::BUSSIM_VEHICLE(int _ID, int _ACTIVE, int _WAITING, double _VELOCITY, double _LAST_UPDATE_TIME, double _POS, double _REMAINING_WAITING_TIME, class BUSSIM_ARC _ON_ARC)
{
    this->ID = _ID;
    this->on_line = -1;
    this->velocity = _VELOCITY;
    this->ACTIVE = _ACTIVE;
    this->WAITING = _WAITING;
    this->VISIBLE = 1;

    this->last_update_time = _LAST_UPDATE_TIME;

    this->ON_ARC = _ON_ARC;

    this->POS = _POS;
    this->remaing_waiting_time = _REMAINING_WAITING_TIME;

    this->km_travelled_on_line = 0;
    this->start_time_on_line = -1.0;

    this->km_travelled_on_arc = 0;
    this->start_time_on_arc = -1.0;

    this->Thwarted = 0;

    this->rotation = nullptr;

    this->next_arc.ID = -1;     // to be set in the activate function
    this->last_visited_node = -1;     // to be set in the activate function
}

BUSSIM_VEHICLE::~BUSSIM_VEHICLE()
{

}

void BUSSIM_VEHICLE::update(class BUSSIM_NETWORK *_NETWORK)
{
    //std::cout << "POS" << this->POS << std::endl;
    if( (_NETWORK->CURRENT_TIME > 0) && (BUSSIM_TIME_STEP > -5 ) )
    {

    if( this->ACTIVE > 0 )
    {
        double add_distance_screen=0.0;;
        // proceeds only for an active vehicle

        if( this->WAITING > 0 )
        {
            this->remaing_waiting_time -= (BUSSIM_TIME_STEP);
            if( this->remaing_waiting_time <= 0)
            {
                // vehicle is ready to go
                this->remaing_waiting_time = 0.0;
                this->WAITING = 0;

                // this is the next served arc
                if( this->ON_ARC.pos < _NETWORK->LINE[this->on_line].ARCS - 1)
                {
                    // end of line has not yet reached: goto next arc
                    //std::cout << "vehicle " << this->ID << "repos A" << std::endl;
                    this->ON_ARC = _NETWORK->LINE[this->on_line].USED_ARC[this->ON_ARC.pos+1];
                    this->km_travelled_on_arc = 0.0;
                    this->start_time_on_arc = _NETWORK->CURRENT_TIME;
                }
                else
                {
                    // the end of a line has reached: goto to subsequent line to be served
                    if( _NETWORK->BETRIEBS_ENDE < 1)
                    {


                        //std::cout << "next line is " << _NETWORK->LINE[this->on_line].ID_NEXT_LINE << std::endl;
                        //if( _NETWORK->LINE[this->on_line].ID_NEXT_LINE < 0 )
                        if( this->rotation->current_duty->next == nullptr )
                        {
                            // no next line found de-active vehicle
                            //std::cout << _NETWORK->CURRENT_TIME << ": no new line for vehicle " << this->ID << "-> deactivated" << std::endl;
                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t no new task\t at end of line %d\t deactivated at stop %d",this->ID,this->on_line,this->ON_ARC.dest_node);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_VEHICLE_DEACTIVATION,TMP);
                            _NETWORK->append_event(NewEvent);
                            NewEvent = nullptr;

                            this->ACTIVE = 0;
                        }
                        else
                        {
                            // set arc to first arc of next service
                            //std::cout << "switch arc" << this->POS << std::endl;
                            //std::cout << "vehicle " << this->ID << "repos B" << std::endl;
                            //this->ON_ARC = _NETWORK->LINE[_NETWORK->LINE[this->on_line].ID_NEXT_LINE].USED_ARC[0];

                            // go to next duty in this rotation
                            this->rotation->current_duty = this->rotation->current_duty->next;

                            this->ON_ARC = _NETWORK->LINE[this->rotation->current_duty->TRIP_ID].USED_ARC[0];

                            this->km_travelled_on_arc = 0.0;
                            this->start_time_on_arc = _NETWORK->CURRENT_TIME;

                            //this->on_line = _NETWORK->LINE[this->on_line].ID_NEXT_LINE;
                            this->on_line = this->rotation->current_duty->TRIP_ID;
                            this->km_travelled_on_line = 0;
                            this->start_time_on_line = _NETWORK->CURRENT_TIME;
                        }
                    }
                    else
                    {
                        // de-active vehicle due to betriebsende

                        char TMP[256];
                        sprintf(TMP,"vehicle\t%d\t end of work\t end of line %d\t deactivated at stop %d",this->ID,this->on_line,this->ON_ARC.dest_node);
                        class BUSSIM_EVENT *NewEvent;
                        NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_VEHICLE_END_OF_WORK,TMP);
                        _NETWORK->append_event(NewEvent);
                        NewEvent = nullptr;

                        this->ACTIVE = 0;
                    }
                }

                // vehicle is at the beginning of the next arc
                this->POS = 0.0;

                char TMP[256];
                sprintf(TMP,"vehicle\t%d\t line\t%d\t stop\t%d",this->ID,this->on_line,this->ON_ARC.orig_node);
                class BUSSIM_EVENT *NewEvent;
                NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_DEPARTURE,TMP);

                /*
                if( this->ON_ARC.orig_node == 10 )
                    if( this->on_line == 0)
                    {
                        std::cout << _NETWORK->CURRENT_TIME << ":" << TMP << " TYP=" << BUSSIM_DEPARTURE << std::endl;
                        NewEvent->PRINT_ON_SCREEN();
                    }
                */
                _NETWORK->append_event(NewEvent);
                NewEvent = nullptr;

            }
        }

        if( this->WAITING < 1)
        {
            // vehicle is moving: update it's position

            // calculate the additional driving distance during _DeltaTime in kilometers
            double add_distance_km;
            //double add_distance_pos;

            add_distance_km = (this->velocity) / 60.0 * (double)(BUSSIM_TIME_STEP);


            // scale it into the screened coordinate system

            add_distance_screen = (add_distance_km - 0) / (_NETWORK->xmax - _NETWORK->xmin) *2.0;

            //std::cout << "_DeltaTime[0]=" << _DeltaTime[0] << std::endl;
            //std::cout << "add_distance_km" << add_distance_km << std::endl;
            //std::cout << "xmin" << _NETWORK->xmin << std::endl;
            //std::cout << "xmax" << _NETWORK->xmax << std::endl;
            //std::cout << "ymin" << _NETWORK->ymin << std::endl;
            //std::cout << "ymax" << _NETWORK->ymax << std::endl;
            // determine the new pos-vector
            double new_pos_vec_x;
            double new_pos_vec_y;

            new_pos_vec_x = this->POS*(this->ON_ARC.dest_screen_x - this->ON_ARC.orig_screen_x) + (this->ON_ARC.dest_screen_x - this->ON_ARC.orig_screen_x) / (double)(this->ON_ARC.length_screen)*add_distance_screen;
            new_pos_vec_y = this->POS*(this->ON_ARC.dest_screen_y - this->ON_ARC.orig_screen_y) + (this->ON_ARC.dest_screen_y - this->ON_ARC.orig_screen_y) / (double)(this->ON_ARC.length_screen)*add_distance_screen;

            // determine length of new pos-vector
            double new_pos_vec_length;
            new_pos_vec_length = sqrt(pow(new_pos_vec_x,2)+pow(new_pos_vec_y,2));


            // now check if vehicle has reached destination of ON_ARC
            if( (new_pos_vec_length / this->ON_ARC.length_screen) < 1 )
            {
                // not yet

                // check if repos is possible without overtaking
                int overtake=0;

                //add_distance_pos = (new_pos_vec_length / this->ON_ARC.length_screen);
                //overtake = this->check_for_takeover_crossline(_NETWORK,&add_distance_km);
                //overtake += this->check_for_takeover(_NETWORK,&add_distance_km);

                overtake = this->check_for_takeover_new(_NETWORK,&add_distance_km);

                if( overtake == 0 )
                {
                    // repositioning possible
                    this->POS = (new_pos_vec_length / this->ON_ARC.length_screen);
                    // save the travel distance increase of this vehicle
                    this->km_travelled_on_line += add_distance_km;

                    // reset the on_arc-properties
                    this->km_travelled_on_arc += add_distance_km;
                }
                else
                {
                    // vehicle cannot move forward
                    //std::cout << "vehicle " << this->ID << "stopped by previous vehicle " << std::endl;
                    //AUTOMATIC_MODE = 0;

                    class BUSSIM_BUNCHPOINT *A;
                    A = new class BUSSIM_BUNCHPOINT;
                    A->screen_x = new_pos_vec_x + this->ON_ARC.orig_screen_x;
                    A->screen_y = new_pos_vec_y + this->ON_ARC.orig_screen_y;

                    _NETWORK->append_bunch_point(A);
                    delete A;

                }
            }
            else
            {
                // has reached end on arc
                //std::cout << "time " << _NETWORK->CURRENT_TIME <<": arrival of vehicle " << this->ID << " at stop " << this->ON_ARC.dest_node << " serving line " << this->on_line << std::endl;

                // update the last stop information
                this->next_arc = _NETWORK->ARC[this->ON_ARC.dest_node];
                this->last_visited_node = this->ON_ARC.dest_node;
                this->last_node_arrival_time = _NETWORK->CURRENT_TIME + (double)(this->ID)/10000.0;;

                char TMP[256];
                sprintf(TMP,"vehicle\t%d\t line\t%d\t stop\t%d",this->ID,this->on_line,this->ON_ARC.dest_node);
                class BUSSIM_EVENT *NewEvent;
                NewEvent = new class BUSSIM_EVENT(_NETWORK->CURRENT_TIME,BUSSIM_ARRIVAL,TMP);
                _NETWORK->append_event(NewEvent);
                NewEvent = nullptr;

                // check if repos is possible without overtaking
                int overtake=0;

                //add_distance_pos = 0;
                //overtake = this->check_for_takeover_crossline(_NETWORK,&add_distance_km);
                //overtake += this->check_for_takeover(_NETWORK,&add_distance_km);

                overtake = this->check_for_takeover_new(_NETWORK,&add_distance_km);

                if( overtake == 0 )
                {
                    this->POS = 1.0;
                    // set the waiting time ...
                    this->remaing_waiting_time = BUSSIM_MEAN_WAITING_TIME - BUSSIM_DEVIATION_WAITING_TIME + (double)(rand() % 1001)/1000.0*2.0*BUSSIM_DEVIATION_WAITING_TIME;
                    // ... and let the vehicle wait
                    this->WAITING = 1;
                    // save the travel distance increase of this vehicle
                    this->km_travelled_on_line += add_distance_km;
                    this->km_travelled_on_arc += add_distance_km;
                }
                else
                {
                    // vehicle cannot move forward
                    //std::cout << "vehicle " << this->ID << "stopped by previous vehicle " << std::endl;
                    //AUTOMATIC_MODE = 0;


                    class BUSSIM_BUNCHPOINT *A;
                    A = new class BUSSIM_BUNCHPOINT;
                    A->screen_x = new_pos_vec_x + this->ON_ARC.orig_screen_x;
                    A->screen_y = new_pos_vec_y + this->ON_ARC.orig_screen_y;

                    _NETWORK->append_bunch_point(A);
                    delete A;

                }
            }
        }
        //std::cout << "distance on screen: " << add_distance_screen << std::endl;
        //std::cout << "pos: " << this->POS << std::endl;
        //std::cout << "on_arc:" << this->ON_ARC.orig_node << ":" << this->ON_ARC.dest_node << std::endl;
        //std::cout << "remaining waiting time:" << this->remaing_waiting_time << ":" << std::endl;
        this->last_update_time += BUSSIM_TIME_STEP;
    }
    }
}

void BUSSIM_VEHICLE::print_on_screen(class BUSSIM_NETWORK *_NET)
{

   if( this->on_line >= 0)
        if( _NET->LINE[this->on_line].displayed > 0)
    if( this->ACTIVE > 0 )
    {


    float start_x;
    float start_y;

    start_x = _NET->LINE[this->on_line].x_offset + _NET->PT[this->ON_ARC.orig_node].screen_x + this->POS * (_NET->PT[this->ON_ARC.dest_node].screen_x - _NET->PT[this->ON_ARC.orig_node].screen_x);
    start_y = _NET->LINE[this->on_line].y_offset + _NET->PT[this->ON_ARC.orig_node].screen_y + this->POS * (_NET->PT[this->ON_ARC.dest_node].screen_y - _NET->PT[this->ON_ARC.orig_node].screen_y);

    glColor4f(_NET->LINE[this->on_line].RED, _NET->LINE[this->on_line].GREEN, _NET->LINE[this->on_line].BLUE, 0.0f);

    double r = 0.015;
    int num_segments = 1000;

    glBegin(GL_LINE_LOOP);
    //glColor4f(0.0f, 1.0f, 0.0f, 0.0f);
    for(int ii = 0; ii < num_segments; ii++)
    {
        float theta = 2.0f * 3.1415926f * float(ii) / float(num_segments);//get the current angle

        float x = r * cosf(theta);//calculate the x component
        float y = r * sinf(theta);//calculate the y component

        glVertex2f(start_x, start_y);
        glVertex2f(start_x + x, start_y+ y);//output vertex

    }
    //glColor4f(1.0f, 0.0f, 0.0f, 0.0f);
    glEnd();
    glColor4f(_NET->LINE[this->on_line].RED, _NET->LINE[this->on_line].GREEN, _NET->LINE[this->on_line].BLUE, 0.0f);
    //glRasterPos2d(start_x+2*_NET->LINE[this->on_line].x_offset, start_y+1*_NET->LINE[this->on_line].y_offset);
    glRasterPos2d(start_x+0.02, start_y-0.01);

    char S[256];
    if(this->WAITING < 1)
        if( _NET->LINE[this->on_line].type == BUSSIM_REGULARLINE )
       //sprintf(S,"V%d (%d POS=%8.2f /ST=%8.2f)->%d (last=%d arr. %.2f)",this->ID, this->ON_ARC.ID, this->POS, this->start_time_on_arc,this->next_arc.ID, this->last_visited_node, this->last_node_arrival_time);
            sprintf(S,"V%d (%.2f km)",this->ID, this->km_travelled_on_line);
        else
            sprintf(S,"V%d (S) (%.2f km)",this->ID, this->km_travelled_on_line);
    else
        sprintf(S,"V%d (w %.1f min)",this->ID,this->remaing_waiting_time);
    unsigned char* ubuf = reinterpret_cast<unsigned char*>(S);
    glutBitmapString(GLUT_BITMAP_HELVETICA_10, ubuf);
    }
}

void BUSSIM_VEHICLE::assign_to_line(class BUSSIM_NETWORK *_NET, class BUSSIM_LINE *_LINE)
{
    this->ACTIVE = 0;
    this->WAITING = 0;
    //this->ID = _LINE->ID;
    this->remaing_waiting_time = 0;
    this->last_update_time = _NET->CURRENT_TIME;
    this->on_line = _LINE->ID;
    this->ON_ARC = _LINE->USED_ARC[0];
    this->POS = 0.0;
    this->km_travelled_on_line = 0;

}

int BUSSIM_VEHICLE::check_for_takeover(class BUSSIM_NETWORK *_NET, double *ADD_DIST)
{
    int ret_val = 0;

    for( int v = 0 ; v < _NET->VEHICLES ; v++ )
    {
        if( _NET->VEHICLE[v].ID != this->ID)
        {
            if( _NET->VEHICLE[v].ACTIVE > 0)
                //if( this->WAITING < 1)
            {
                if( _NET->VEHICLE[v].on_line == this->on_line )
                {
                    // both vehicle are currently service the same line
                    if( _NET->VEHICLE[v].start_time_on_line < this->start_time_on_line )
                    {
                        // vehicle v has started earlier than this ...
                        if( _NET->VEHICLE[v].km_travelled_on_line <= (this->km_travelled_on_line + ADD_DIST[0])  ) // HERE
                        {
                            // ... v has not yet travelled so far
                            ret_val = 1;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT required takeover of vehicle " <<_NET->VEHICLE[v].ID << " by " << this->ID << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT travel distance of vehicle " << _NET->VEHICLE[v].ID << " on this arc so far: " << _NET->VEHICLE[v].km_travelled_on_arc << " (enter time: " << _NET->VEHICLE[v].start_time_on_arc <<")" << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT travel distance of vehicle " << this->ID << " on this arc so far: " << this->km_travelled_on_arc << " (enter time: " << this->start_time_on_arc <<")" << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT required add km " << ADD_DIST[0] << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT on line ("<< this->ON_ARC.orig_node << ";" << this->ON_ARC.dest_node << ") at POS " << this->POS << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": LOT waiting ("<< _NET->VEHICLE[v].WAITING << ";" << this->WAITING << ") at POS " << this->POS << std::endl;

                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                            _NET->append_event(NewEvent);
                            NewEvent = nullptr;

                            _NET->DETECTED_BUNCHES++;
                            this->Thwarted++;
                            v = _NET->VEHICLES+1;
                        }
                    }
                }
            }
        }
    }

    return ret_val;
}

int BUSSIM_VEHICLE::check_for_takeover_crossline(class BUSSIM_NETWORK *_NET, double *ADD_DIST)
{
    int ret_val = 0;

    for( int v = 0 ; v < _NET->VEHICLES ; v++ )
    {
        if( _NET->VEHICLE[v].ID != this->ID)
        {
            if( _NET->VEHICLE[v].ACTIVE > 0)
            if( this->on_line != _NET->VEHICLE[v].on_line )
            {
                if( _NET->VEHICLE[v].ON_ARC.ID == this->ON_ARC.ID )
                {
                    // both vehicle are currently travelling on the same arc
                    if( _NET->VEHICLE[v].start_time_on_arc < this->start_time_on_arc )
                    {
                        // vehicle v has started earlier than this ...
                        if( _NET->VEHICLE[v].km_travelled_on_arc + 0.00001 < (this->km_travelled_on_arc + ADD_DIST[0])  )
                        {
                            // .. v but has not yet travelled as much km as this!
                            ret_val = 1;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": required takeover of vehicle " <<_NET->VEHICLE[v].ID << " by " << this->ID << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << _NET->VEHICLE[v].ID << " on this arc so far: " << _NET->VEHICLE[v].km_travelled_on_arc << " (enter time: " << _NET->VEHICLE[v].start_time_on_arc <<")" << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << this->ID << " on this arc so far: " << this->km_travelled_on_arc << " (enter time: " << this->start_time_on_arc <<")" << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": required add km " << ADD_DIST[0] << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": on shared arc ("<< this->ON_ARC.orig_node << ";" << this->ON_ARC.dest_node << ") at POS " << this->POS << std::endl;
                            //std::cout << "time " << _NET->CURRENT_TIME << ": waiting ("<< _NET->VEHICLE[v].WAITING << ";" << this->WAITING << ") at POS " << this->POS << std::endl;

                            char TMP[256];
                            sprintf(TMP,"vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                            class BUSSIM_EVENT *NewEvent;
                            NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                            _NET->append_event(NewEvent);
                            NewEvent = nullptr;

                            _NET->DETECTED_BUNCHES++;
                            this->Thwarted++;
                            //exit(0);
                            v = _NET->VEHICLES+1;
                        }
                    }
                    else
                    {
                        // hier weiter mit last_node und at_last_node

                    }
                }
            }
       }
    }

    return ret_val;
}

void BUSSIM_VEHICLE::activate(class BUSSIM_NETWORK *_NET)
{
    if( this->ACTIVE < 1)
    {
        if( this->rotation->first_duty != nullptr )
        {
            // there is a rotation available for this vehicle

            this->assign_to_line(_NET,&_NET->LINE[this->rotation->first_duty->TRIP_ID]);

            // save the information of the last stop
            this->next_arc = _NET->ARC[_NET->LINE[this->rotation->first_duty->TRIP_ID].USED_ARC[0].ID];
            this->last_visited_node  = _NET->ARC[_NET->LINE[this->rotation->first_duty->TRIP_ID].USED_ARC[0].ID].orig_node;
            this->last_node_arrival_time = _NET->CURRENT_TIME + (double)(this->ID)/10000.0;

            this->ACTIVE = 1;
            this->start_time_on_line = _NET->CURRENT_TIME;
            this->start_time_on_arc = _NET->CURRENT_TIME;

            // set pointer to the current duty of this vehicles
            this->rotation->current_duty = this->rotation->first_duty;
            char TMP[256];
            sprintf(TMP,"vehicle\t%d\t activated on line %d\t at stop %d",this->ID,_NET->LINE[this->rotation->first_duty->TRIP_ID].ID,_NET->LINE[this->rotation->first_duty->TRIP_ID].USED_ARC[0].ID);
            class BUSSIM_EVENT *NewEvent;
            NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_ACTIVATION,TMP);
            _NET->append_event(NewEvent);
            NewEvent = nullptr;
        }
    }
}


int BUSSIM_VEHICLE::check_for_takeover_new(class BUSSIM_NETWORK *_NET, double *ADD_DIST)
{
    int ret_val;
    ret_val = 0;

    for( int v = 0 ; v < _NET->VEHICLES ; v++ )
    {
        if( _NET->VEHICLE[v].ID != this->ID )
        {
            if( _NET->VEHICLE[v].ACTIVE > 0)
            {
                //if( this->on_line != _NET->VEHICLE[v].on_line )
                if( 1 < 2 )
                {
                    if( _NET->VEHICLE[v].ON_ARC.ID == this->ON_ARC.ID )
                    {

                        _NET->PRED[_NET->VEHICLE[v].ID][this->ID] = 0;

                        // both vehicle are currently travelling on the same arc
                        if( _NET->VEHICLE[v].start_time_on_arc < this->start_time_on_arc )
                        {

                            _NET->PRED[_NET->VEHICLE[v].ID][this->ID] = 1;

                            // vehicle v has started earlier than this ...
                            if( _NET->VEHICLE[v].km_travelled_on_arc + 0.00001 < (this->km_travelled_on_arc + ADD_DIST[0])  )
                            {
                                // .. v but has not yet travelled as much km as this!
                                ret_val = 1;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": required takeover of vehicle " <<_NET->VEHICLE[v].ID << " by " << this->ID << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << _NET->VEHICLE[v].ID << " on this arc so far: " << _NET->VEHICLE[v].km_travelled_on_arc << " (enter time: " << _NET->VEHICLE[v].start_time_on_arc <<")" << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": travel distance of vehicle " << this->ID << " on this arc so far: " << this->km_travelled_on_arc << " (enter time: " << this->start_time_on_arc <<")" << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": required add km " << ADD_DIST[0] << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": on shared arc ("<< this->ON_ARC.orig_node << ";" << this->ON_ARC.dest_node << ") at POS " << this->POS << std::endl;
                                //std::cout << "time " << _NET->CURRENT_TIME << ": waiting ("<< _NET->VEHICLE[v].WAITING << ";" << this->WAITING << ") at POS " << this->POS << std::endl;

                                char TMP[256];
                                sprintf(TMP,"vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                                class BUSSIM_EVENT *NewEvent;
                                NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                                _NET->append_event(NewEvent);
                                NewEvent = nullptr;

                                _NET->DETECTED_BUNCHES++;
                                this->Thwarted++;
                                //exit(0);
                                v = _NET->VEHICLES+1;
                            }
                        }
                        else
                        {
                            // hier weiter mit last_node und at_last_node

                        }
                    }
                    else
                    {
                        // the two vehicle are not on the same arc: check if they will be one the same arc in a moment
                        if( _NET->VEHICLE[v].next_arc.ID == this->next_arc.ID )
                        {
                            if( _NET->VEHICLE[v].next_arc.ID >= 0 )
                            {
                                if( this->next_arc.ID >= 0)
                                {
                                    // both vehicles have been busy before
                                    if( _NET->VEHICLE[v].last_visited_node == this->last_visited_node )
                                    {
                                        // if( _NET->VEHICLE[v].last_node_arrival_time  <= this->last_node_arrival_time )
                                        {
                                            // falls gleich: immer nach vehicle ID
                                            // vehicle v arrived before this vehicle at last common stop
                                            if( _NET->VEHICLE[v].POS >= 1.0 )
                                            {
                                                // vehicle v has not yet left node last_visisted_node
                                                if( this->POS < 0.5 )
                                                if( _NET->PRED[_NET->VEHICLE[v].ID][this->ID] > 0 )
                                                {
                                                    // this vehicle is going to leave the node last_visisted node :-(
                                                    ret_val = 1;

                                                    char TMP[256];
                                                    sprintf(TMP,"**vehicle\t%d\t on line\t %d\t is blocked by vehicle\t%d\t on line\t%d\t both on arc\t%d",this->ID,this->on_line,_NET->VEHICLE[v].ID,_NET->VEHICLE[v].on_line,this->ON_ARC.ID);
                                                    class BUSSIM_EVENT *NewEvent;
                                                    NewEvent = new class BUSSIM_EVENT(_NET->CURRENT_TIME,BUSSIM_VEHICLE_BLOCKAGE,TMP);
                                                    _NET->append_event(NewEvent);
                                                    NewEvent = nullptr;

                                                    _NET->DETECTED_BUNCHES++;
                                                    this->Thwarted++;
                                                    //exit(0);
                                                    v = _NET->VEHICLES+1;
                                                }
                                            }
                                        }

                                    }
                                }

                            }
                        }
                    }
                }
            }
        }
    }

    return ret_val;
}
