#ifndef BUSSIM_VEHICLE_H_INCLUDED
#define BUSSIM_VEHICLE_H_INCLUDED

#include "bussim_network.h"

class BUSSIM_VEHICLE {
public:
    int ID;             // unique identifier for this vehicle

    int on_line;        // refers to the currently served line

    double velocity;    // given in km/h

    int ACTIVE;         // if true (=1) then this vehicle is updated otherwise (=0) updates are not made
    int WAITING;          // 1 = TRAVELLING, 2 = WAITING

    double last_update_time;        // time point with last update
    double remaing_waiting_time;    // remaining waiting time at a stop

    class BUSSIM_ARC ON_ARC;         // gives the identifier of an arc
    double POS;         // position on ON_ARC, varies between 0 (at arc origin) and 1 (at arc destination)

    double km_travelled_on_line;
    double start_time_on_line;

    double km_travelled_on_arc;     // needed to find out if overtaking is needed on an arc
    double start_time_on_arc;

    int Thwarted;        // number of events where a vehicle is blocked by another

    int VISIBLE;

    struct BUSSIM_ROTATION *rotation;   // reference to the vehicle's rotation

    struct BUSSIM_ARC next_arc;     // what is the next traversed arc (initialized by -1)
    int last_visited_node;          // what was the last visited node? (initialized by -1);
    double last_node_arrival_time;  // when did it arrived at last_visited_node?;

    BUSSIM_VEHICLE();
    BUSSIM_VEHICLE(int _ID, int _ACTIVE, int _WAITING, double _VELOCITY, double _LAST_UPDATE_TIME, double _POS, double _REMAINING_WAITING_TIME, class BUSSIM_ARC _ON_ARC);
    ~BUSSIM_VEHICLE();

    void configure(int _ID, double _VELOCITY, class BUSSIM_NETWORK *_NET);
    void update(class BUSSIM_NETWORK *_NETWORK);
    void print_on_screen(class BUSSIM_NETWORK *_NET);
    void assign_to_line(class BUSSIM_NETWORK *_NET, class BUSSIM_LINE *_LINE);
    int check_for_takeover(class BUSSIM_NETWORK *_NET, double *ADD_DIST);
    int check_for_takeover_crossline(class BUSSIM_NETWORK *_NET, double *ADD_DIST);
    int check_for_takeover_new(class BUSSIM_NETWORK *_NET, double *ADD_DIST);
    void activate(class BUSSIM_NETWORK *_NET);
};

#endif // BUSSIM_VEHICLE_H_INCLUDED
